## Accompanying materials to the Online Appendix to
## "Private returns to R\&D in the presence of spillovers, revisited"
## Forthcoming, Journal of Applied Econometrics
## Giovanni Millo, September 14th 2018

## Modified pgmm function from observations of Nina Schoenfelder,
## fixes some erratic behaviour in ill-conditioned problems.

mypgmm<-function (formula, data, subset, na.action, effect = c("twoways",
    "individual"), model = c("onestep", "twosteps"), collapse = FALSE,
    lost.ts = NULL, transformation = c("d", "ld"), fsm = NULL,
    index = NULL, ...)
{
    cl <- match.call(expand.dots = FALSE)
    effect <- match.arg(effect)
    model <- match.arg(model)
    transformation <- match.arg(transformation)
    namesV <- NULL
    if (inherits(formula, "dynformula") || length(Formula(formula))[2] ==
        1) {
        if (!inherits(formula, "dynformula")) {
            formula <- match.call(expand.dots = TRUE)
            m <- match(c("formula", "lag.form", "diff.form",
                "log.form"), names(formula), 0)
            formula <- formula[c(1, m)]
            formula[[1]] <- as.name("dynformula")
            formula <- cl$formula <- eval(formula, parent.frame())
        }
        response.name <- paste(deparse(formula[[2]]))
        main.lags <- attr(formula, "lag")
        if (length(main.lags[[1]]) == 1 && main.lags[[1]] > 1)
            main.lags[[1]] <- c(1, main.lags[[1]])
        main.lags[2:length(main.lags)] <- lapply(main.lags[2:length(main.lags)],
            function(x) {
                if (length(x) == 1 && x != 0)
                  x <- c(0, x)
                x
            })
        main.form <- dynterms2formula(main.lags, response.name)
        dots <- list(...)
        gmm.inst <- dots$gmm.inst
        lag.gmm <- dots$lag.gmm
        instruments <- dots$instruments
        gmm.form <- dynformula(gmm.inst, lag.form = lag.gmm)
        gmm.lags <- attr(gmm.form, "lag")
        gmm.lags <- lapply(gmm.lags, function(x) min(x):max(x))
        gmm.form <- dynterms2formula(gmm.lags)
        formula <- as.Formula(main.form, gmm.form)
    }
    x <- formula
    if (!inherits(x, "Formula"))
        x <- Formula(formula)
    gmm.form <- formula(x, rhs = 2, lhs = 0)
    gmm.lags <- dynterms(gmm.form)
    cardW <- length(gmm.lags)
    if (is.null(names(collapse))) {
        if (length(collapse) == 1) {
            collapse <- as.vector(rep(collapse, cardW), mode = "list")
        }
        else {
            if (length(collapse) != cardW)
                stop("the collapse vector has a wrong lenght")
        }
        names(collapse) <- names(gmm.lags)
    }
    else {
        if (any(!(names(collapse) %in% names(gmm.lags))))
            stop("unknown names in the collapse vector")
        else {
            bcollapse <- as.vector(rep(FALSE, cardW), mode = "list")
            names(bcollapse) <- names(gmm.lags)
            bcollapse[names(collapse)] <- collapse
            collapse <- bcollapse
        }
    }
    main.form <- formula(x, rhs = 1, lhs = 1)
    main.lags <- dynterms(main.form)
    if (length(x)[2] == 3) {
        normal.instruments <- TRUE
        inst.form <- formula(x, rhs = 3, lhs = 0)
        inst.form <- update(main.form, inst.form)
        inst.form <- formula(Formula(inst.form), lhs = 0)
        inst.lags <- dynterms(inst.form)
    }
    else {
        iv <- names(main.lags)[!names(main.lags) %in% names(gmm.lags)]
        inst.lags <- main.lags[iv]
        if (length(inst.lags) > 0) {
            normal.instruments <- TRUE
            inst.form <- dynterms2formula(inst.lags)
        }
        else {
            normal.instruments <- FALSE
            inst.form <- NULL
            inst.lags <- NULL
        }
    }
    if (!is.null(lost.ts)) {
        if (!is.numeric(lost.ts))
            stop("lost.ts should be numeric")
        lost.ts <- as.numeric(lost.ts)
        if (!(length(lost.ts) %in% c(1, 2)))
            stop("lost.ts should be of length 1 or 2")
        TL1 <- lost.ts[1]
        TL2 <- ifelse(length(lost.ts == 1), TL1 - 1, lost.ts[2])
    }
    else {
        gmm.minlag <- max(sapply(gmm.lags, min))
        gmm.minlag <- min(sapply(gmm.lags, min))
        if (!is.null(inst.lags))
            inst.maxlag <- max(sapply(inst.lags, max))
        else inst.maxlag <- 0
        main.maxlag <- max(sapply(main.lags, max))
        TL1 <- max(main.maxlag + 1, inst.maxlag + 1, gmm.minlag)
        TL2 <- max(main.maxlag, inst.maxlag, gmm.minlag - 1)
        TL1 <- max(main.maxlag + 1, gmm.minlag)
        TL2 <- max(main.maxlag, gmm.minlag - 1)
    }
    gmm.form <- as.formula(paste("~", paste(names(gmm.lags),
        collapse = "+")))
    if (!is.null(inst.form))
        Form <- as.Formula(main.form, gmm.form, inst.form)
    else Form <- as.Formula(main.form, gmm.form)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "na.action", "index"),
        names(mf), 0)
    mf <- mf[c(1, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1]] <- as.name("plm")
    mf$model <- NA
    mf$formula <- Form
    mf$na.action <- "na.pass"
    mf$subset <- NULL
    data <- eval(mf, parent.frame())
    index <- index(data)
    N <- length(levels(index[[1]]))
    T <- length(levels(index[[2]]))
    pdim <- pdim(data)
    balanced <- pdim$balanced
    if (!balanced) {
        un.id <- sort(unique(index(data, "id")))
        un.time <- sort(unique(index(data, "time")))
        rownames(data) <- paste(index(data, "id"), index(data,
            "time"), sep = ".")
        allRows <- as.character(t(outer(un.id, un.time, paste,
            sep = ".")))
        data <- data[allRows, ]
        rownames(data) <- allRows
        index <- data.frame(id = rep(un.id, each = length(un.time)),
            time = rep(un.time, length(un.id)), row.names = rownames(data))
        class(index) <- c("pindex", "data.frame")
        attr(data, "index") <- index
    }
    attr(data, "formula") <- formula(main.form)
    yX <- extract.data(data)
    names.coef <- colnames(yX[[1]])[-1]
    if (normal.instruments) {
        attr(data, "formula") <- inst.form
        Z <- extract.data(data)
    }
    else Z <- NULL
    attr(data, "formula") <- gmm.form
    W <- extract.data(data, as.matrix = FALSE)
    W1 <- lapply(W, function(x) {
        u <- mapply(mymakegmm, x, gmm.lags, TL1, collapse, SIMPLIFY = FALSE)
        u <- matrix(unlist(u), nrow = nrow(u[[1]]))
        u
    })
    yX1 <- lapply(yX, function(x) {
        xd <- diff(x)
        xd <- xd[-c(1:(TL1 - 1)), , drop = FALSE]
        xd
    })
    if (normal.instruments) {
        Z1 <- lapply(Z, function(x) {
            xd <- diff(x)
            xd <- xd[-c(1:(TL1 - 1)), , drop = FALSE]
            xd
        })
    }
    if (transformation == "ld") {
        W2 <- lapply(W, function(x) {
            u <- mapply(mymakeW2,x, collapse, SIMPLIFY = FALSE)## Modifications here
            nrow.ud <- ifelse(TL2 == 1, T - 2, T - TL2)
            u <- matrix(unlist(u), nrow = nrow.ud)
            if (TL2 == 1)
                u <- rbind(0, u)
            u
        })
        yX2 <- lapply(yX, function(x) {
            x <- x[-c(0:TL2), , drop = FALSE]
            x
        })
        if (normal.instruments) {
            Z2 <- lapply(Z, function(x) {
                x <- x[-c(0:TL2), , drop = FALSE]
                x
            })
        }
    }
    if (effect == "twoways") {
        namesV <- levels(index(data, which = "time"))
        if (transformation == "d") {
            V1 <- td.model.diff <- diff(diag(1, T - TL1 + 1))[,
                -1]
            namesV <- namesV[-c(0:(TL1))]
        }
        else {
            td <- cbind(1, rbind(0, diag(1, T - 1)))
            V2 <- td[-c(1:TL2), -c(2:(2 + TL2 - 1))]
            V1 <- diff(V2)
            namesV <- c("(intercept)", namesV[-c(0:TL2 + 1)])
        }
        for (i in 1:N) {
            yX1[[i]] <- cbind(yX1[[i]], V1)
            if (transformation == "d") {
                W1[[i]] <- cbind(W1[[i]], V1)
            }
            else {
                W2[[i]] <- cbind(W2[[i]], V2)
                yX2[[i]] <- cbind(yX2[[i]], V2)
            }
        }
    }
    if (effect == "individual" && transformation == "ld") {
        namesV <- levels(index(data, which = "time"))
        namesV <- c("(intercept)", namesV[-c(0:TL2 + 1)])
    }
    for (i in 1:N) {
        narows <- apply(yX1[[i]], 1, function(z) any(is.na(z)))
        yX1[[i]][narows, ] <- 0
        W1[[i]][is.na(W1[[i]])] <- 0
        W1[[i]][narows, ] <- 0
        if (normal.instruments) {
            Z1[[i]][is.na(Z1[[i]])] <- 0
            Z1[[i]][narows, ] <- 0
        }
        if (transformation == "ld") {
            narows <- apply(yX2[[i]], 1, function(z) any(is.na(z)))
            yX2[[i]][narows, ] <- 0
            W2[[i]][is.na(W2[[i]])] <- 0
            W2[[i]][narows, ] <- 0
            if (normal.instruments) {
                Z2[[i]][is.na(Z2[[i]])] <- 0
                Z2[[i]][narows, ] <- 0
            }
        }
    }
    if (transformation == "ld") {
        for (i in 1:N) {
            W1[[i]] <- bdiag(W1[[i]], W2[[i]])
            yX1[[i]] <- rbind(yX1[[i]], yX2[[i]])
            if (normal.instruments)
                Z1[[i]] <- bdiag(Z1[[i]], Z2[[i]])
        }
    }
    if (normal.instruments) {
        for (i in 1:N) W1[[i]] <- cbind(W1[[i]], Z1[[i]])
    }
    W <- W1
    yX <- yX1
    if (transformation == "d")
        A1 <- tcrossprod(diff(diag(1, T - TL1 + 1)))
    if (transformation == "ld")
        A1 <- FSM(T - TL2, "full")
    WX <- mapply(function(x, y) crossprod(x, y), W, yX, SIMPLIFY = FALSE)
    Wy <- lapply(WX, function(x) x[, 1])
    WX <- lapply(WX, function(x) x[, -1])
    A1 <- lapply(W, function(x) crossprod(t(crossprod(x, A1)),
        x))
    A1 <- Reduce("+", A1)
    minevA1 <- min(eigen(A1)$values)
    eps <- 1e-09
    if (minevA1 < eps) {
        A1 <- ginv(A1) * length(W)
        warning("the first-step matrix is singular, a general inverse is used")
    }
    else A1 <- solve(A1) * length(W)
    WX <- Reduce("+", WX)
    Wy <- Reduce("+", Wy)
    B1 <- solve(crossprod(WX, t(crossprod(WX, A1))))
    Y1 <- crossprod(t(crossprod(WX, A1)), Wy)
    coefficients <- as.numeric(crossprod(B1, Y1))
    if (effect == "twoways")
        names.coef <- c(names.coef, namesV)
    names(coefficients) <- names.coef
    residuals <- lapply(yX, function(x) as.vector(x[, 1] - crossprod(t(x[,
        -1]), coefficients)))
    outresid <- lapply(residuals, function(x) outer(x, x))
    A2 <- mapply(function(x, y) crossprod(t(crossprod(x, y)),
        x), W, outresid, SIMPLIFY = FALSE)
    A2 <- Reduce("+", A2)
    minevA2 <- min(eigen(A2)$values)
    eps <- 1e-09
    if (minevA2 < eps) {
        A2 <- ginv(A2)
        warning("the second-step matrix is singular, a general inverse is used")
    }
    else A2 <- solve(A2)
    B2 <- solve(crossprod(WX, t(crossprod(WX, A2))))
    if (model == "twosteps") {
        coef1s <- coefficients
        Y2 <- crossprod(t(crossprod(WX, A2)), Wy)
        coefficients <- as.numeric(crossprod(B2, Y2))
        names(coefficients) <- c(names.coef)
        vcov <- B2
    }
    else vcov <- B1
    rownames(vcov) <- colnames(vcov) <- c(names.coef)
    residuals <- lapply(yX, function(x) {
        nz <- rownames(x)
        z <- as.vector(x[, 1] - crossprod(t(x[, -1]), coefficients))
        names(z) <- nz
        z
    })
    fitted.values <- mapply(function(x, y) x[, 1] - y, yX, residuals)
    if (model == "twosteps")
        coefficients <- list(coef1s, coefficients)
    args <- list(model = model, effect = effect, transformation = transformation,
        namest = namesV)
    result <- list(coefficients = coefficients, residuals = residuals,
        vcov = vcov, fitted.values = fitted.values, df.residual = df.residual,
        model = yX, W = W, A1 = A1, A2 = A2, call = cl, args = args)
    result <- structure(result, class = c("pgmm", "panelmodel"),
        pdim = pdim)
    result
}

env<-environment(plm:::makegmm)
environment(mypgmm)<-env


mymakegmm<-function (x, g, TL1, collapse = FALSE) # Modification here.
{
    T <- length(x)
    rg <- range(g)
    z <- as.list((TL1 + 1):T)
    x <- lapply(z, function(y) x[max(1, y - rg[2]):(y - rg[1])])
    if (collapse) {
	x<-lapply(x, rev)  # Hier unkompliziert aus einer Liste eine gleich aussehende Matrix machen
	m <- matrix(0, T - TL1, min(T - rg[1], rg[2]+1-rg[1]))
	for(y in 1:length(x)){ m[y,1:length(x[[y]])]<-x[[y]]}
	result<-m
    }    else {
        lx <- sapply(x, length)
        n <- length(x)
        lxc <- cumsum(lx)
        before <- c(0, lxc[-n])
        after <- lxc[n] - sapply(x, length) - before
        result <- t(mapply(function(x, y, z) c(rep(0, y), x,
            rep(0, z)), x, before, after, SIMPLIFY = TRUE))
    }
    result
}

env<-environment(plm:::makegmm)
environment(mymakegmm)<-env


mymakeW2<-function (x, collapse = FALSE) ## new function
{
      if (collapse) {
	u<-diff(x[-c(length(x))])

    }    else {
        u<-diag(diff(x[-c(length(x))]))
    }
    u
}

env<-environment(plm:::makegmm)
environment(mymakeW2)<-env
